      Subroutine M05c(F,D1F,RA,RB,D1RA,D1RB,TA,TB,NGrid,ijzy)

************************************************************************
*                                                                      *
*  M06c evaluates the correlation part of the M06 suite of             *
*  functionals on the grids.                                           *
*  !!! Second derivatives are not available yet.                       *
*                                                                      *
*     *  Ref: (a) Zhao, Y., Schultz, N. E., and Truhlar, D. G. J.      *
*     *   Chem. Phys. 123, 161103 (2005).                              *
*     *       (b) Y. Zhao, Schultz, N. E., and D. G. Truhlar, J. Chem. *
*     *   Theory Comput. 2, 364 (2006)                                 * 
*     *                                                                *
*     *       ijzy - 1 M05                                             *
*     *       ijzy - 2 M05-2X                                          *
*     *                                                                *
*                                                                      *
*     *  OUTPUT:                                                       *
*     *     F      - Functional values                                 *
*     *     D1F    - First derivatives with respect to RA, RB, GA, GB  *
*     *              TA, TB                                            *
*     *                                                                *
*     *  INPUT:                                                        *
*     *     RA,B   - Spin densities                                    *
*     *     D1RA,B - Spin density gradients                            *
*     *     TA,B   - Spin kinetic energy densities                     *
*     *     NGrid  - number of grids                                   *
*                                                                      *
*  YZ (12/08)                                                          *
*                                                                      *
************************************************************************
      IMPLICIT NONE

      INTEGER NGrid
      REAL*8  F(NGrid),D1F(NGrid,7),RA(NGrid),RB(NGrid),
     $        D1RA(NGrid,3),D1RB(NGrid,3),TA(NGrid),TB(NGrid)
      REAL*8 Pi, F6, F43, Pi34, F13, 
     &RS,RSP,Zeta,dZdA,dZdB,PotLC,dLdS,dLdZ,d2LdSS,d2LdSZ,d2LdZZ,
     &  P, EUEG,Denom, DenPA, DenPB, DenGA, DenGB
      REAL*8 PA,GAA,TauA,FA,FPA,FGA,FTA,EUA,EUEGA,ChiA,EUPA,ChiAP,ChiAG
      REAL*8 PB,GBB,TauB,FB,FPB,FGB,FTB,EUB,EUEGB,ChiB,EUPB,ChiBP,ChiBG
      REAL*8 sopp0, sopp1,sopp2, sopp3, sopp4
      REAL*8 U, W, dUdChiA,dUdChiB,dUdPA,dUdPB,dUdGA,dUdGB,
     &dWdU,dWdPA,dWdPB, dWdGA,dWdGB,EUEGPA,EUEGPB
      Integer dRA, dRB, dTA, dTB, dGA, dGB
      
      REAL*8  DTol,F1, F2, F3, F4, COpp
      Data COpp/0.0031d0/,F1/1.0d0/,F2/2.0d0/,F3/3.0d0/,F4/4.0d0/ 

      INTEGER i,j,ijzy

      if (ijzy.eq.1) then
C   Parameters for M05 Correlation
         sopp0= 1.00000d0
         sopp1= 3.78569d0
         sopp2= -14.15261d0
         sopp3= -7.46589d0
         sopp4= 17.94491d0
      elseif (ijzy.eq.2) then
C   Parameters for M05-2X Correlation
         sopp0= 1.00000
         sopp1= 1.09297
         sopp2= -3.79171
         sopp3= 2.82810
         sopp4= -10.58909
      endif


      DTol =  1.0d-8
      Pi = F4*ATan(F1)
      F6=6.0d0
      F43 = F4 / F3
      Pi34 = F3 / (F4*Pi)
      F13 = F1 / F3

      DO i = 1,NGrid
         PA = RA(i)
         IF (PA.gt.DTol.and.TA(i).gt.DTol) THEN
            GAA   =  D1RA(i,1)**2 + D1RA(i,2)**2 + D1RA(i,3)**2
            TauA = TA(i)

            call m05css(DTol,PA,GAA,TauA,FA,FPA,FGA,FTA,EUA,
     &                ChiA,EUPA,ChiAP,ChiAG,ijzy)
            F(i) = F(i) + FA
            D1F(i,dRA)  = D1F(i,dRA) + FPA
            D1F(i,dGA) = D1F(i,dGA) + FGA
            D1F(i,dTA)  = D1F(i,dTA) + FTA
         ENDIF
         PB = RB(i)
         IF (PB.gt.DTol.and.TB(i).gt.DTol) THEN
            GBB   =  D1RB(i,1)**2 + D1RB(i,2)**2 + D1RB(i,3)**2
            TauB = TB(i)

            call m05css(DTol,PB,GBB,TauB,FB,FPB,FGB,FTB,EUB,
     &                ChiB,EUPB,ChiBP,ChiBG,ijzy)
            F(i) = F(i) + FB
            D1F(i,dRB)  = D1F(i,dRB) + FPB
            D1F(i,dGB) = D1F(i,dGB) + FGB
            D1F(i,dTB)  = D1F(i,dTB) + FTB
         ENDIF
         P=PA+PB
         IF (PB.gt.DTol.and.PA.gt.DTol) THEN
          RS = (Pi34/P) ** F13 
          RSP = -RS/(F3*P)
          Zeta = (PA-PB)/P
          dZdA = (F1-Zeta)/P
          dZdB = (-F1-Zeta)/P
c       
c      lsdac is a subroutine to evaluate the Perdew-Wang-91 correlation functional 
c      local spin density approximation (LSDA) to the correlation energy of a uniform 
c      electron gas. (Phys. Rev. B 45, 13244 (1992)). Users should provid their own
c      for this LSDA correlation functional or they may find this routine on Kieron 
c      Burke's Web site at http://www.chem.uci.edu/~kieron/dftold2/pubs/PBE.asc
c
          Call lsdac(RS,Zeta,PotLC,dLdS,dLdZ)
          EUEG = P*PotLC - EUA - EUB
          U = COpp*(ChiA+ChiB)/(F1 + COpp*(ChiA+ChiB))
          W = sopp0+U*(sopp1+U*(sopp2+U*(sopp3+U*sopp4)))
          F(i) = F(i) + EUEG*W
          dUdChiA =COpp/(F1 + COpp*(ChiA+ChiB))**2
          dUdChiB =COpp/(F1 + COpp*(ChiA+ChiB))**2
          dUdPA= dUdChiA*ChiAP
          dUdPB= dUdChiB*ChiBP
          dUdGA= dUdChiA*ChiAG
          dUdGB= dUdChiB*ChiBG
          dWdU =sopp1+U*(F2*sopp2+U*(F3*sopp3+U*F4*sopp4))
          dWdPA= dWdU*dUdPA
          dWdPB= dWdU*dUdPB
          dWdGA= dWdU*dUdGA
          dWdGB= dWdU*dUdGB
          EUEGPA = PotLC + P*dLdS*RSP + P*dLdZ*dZdA - EUPA
          EUEGPB = PotLC + P*dLdS*RSP + P*dLdZ*dZdB - EUPB
          D1F(i,dRA)= D1F(i,dRA)+EUEGPA*W + EUEG*dWdPA 
          D1F(i,dGA) = D1F(i,dGA)+EUEG*dWdGA 
          D1F(i,dRB)  = D1F(i,dRB)+EUEGPB*W + EUEG*dWdPB  
          D1F(i,dGB) = D1F(i,dGB)+EUEG*dWdGB
         ENDIF
        ENDDO
        
        RETURN
        END 


      Subroutine m05css(DTol,PX,GX,TX,F,FP,FG,FT,EUEG,Chi,EUEGP,
     &                   ChiP,ChiG,ijzy)
      Implicit none
C
C     Compute the same-spin part of the m05 correlation functional for one grid
C     point and one spin-case.
C
C
      integer ijzy
      double precision PX, GX, TX, F, FP, FG, FT, DTol
      double precision EUEG, Chi, EUEGP, ChiP, ChiG
      double precision Zero, Pt25, F1, F2, F3, F4, F5, F6, F8, F11
      double precision sss0,sss1, sss2, sss3, sss4, Css
      double precision Pi, Pi34, F13, F23, F43, F53, F83, F113
      double precision RS, FDUEG, D, Fscc, RSP, dFsccP, dFsccG
      double precision E, W, U, dFsccT, dUdChi, dWdU, dWdP, dWdG 
      double precision d2LdSS,d2LdSZ,d2LdZZ,PotLC,dLdS,dLdZ
     


      Data Zero/0.0d0/, Pt25/0.25d0/, F1/1.0d0/, F2/2.0d0/, F3/3.0d0/,
     $  F4/4.0d0/, F5/5.0d0/, F6/6.0d0/, F8/8.0d0/, F11/11.0d0/,
     $  Css/0.06d0/
C
c      DTol=1.0D-7
c      write (*,*) DTol
      if (ijzy.eq.1) then
C     Parameters for M05 Correlation
         sss0=  1.00000d0
         sss1=  3.77344d0
         sss2=  -26.04463d0
         sss3=  30.69913d0
         sss4=  -9.22695d0
      elseif (ijzy.eq.2) then
C     Parameters for M05-2X Correlation
         sss0=  1.00000d0
         sss1=  -3.05430d0
         sss2=  7.61854d0
         sss3=  1.47665d0
         sss4=  -11.92365d0
      endif
      
      If ((PX.le.DTol))  then
        EUEG = Zero
        Chi = Zero
        EUEGP = Zero
        ChiP = Zero
        ChiG = Zero
        PX = Zero
        GX = Zero 
        TX = Zero
        F  = Zero
        FP = Zero
        FG = Zero
        FT = Zero
      else
        Pi = F4*ATan(F1)
        Pi34 = F3 / (F4*Pi)
        F13 = F1 / F3
        F23 = F2 / F3
        F43 = F2 * F23
        F53 = F5 / F3
        F83 = F8 / F3
        F113 = F11 / F3
        FDUEG = (F3/F5)*(F6*Pi*Pi)**F23
        RS = (Pi34/PX) ** F13
        Call lsdac(RS,F1,PotLC,dLdS,dLdZ,d2LdSS,d2LdSZ,d2LdZZ)
        EUEG = PX*PotLC
        D = TX - Pt25*GX/PX
C        DUEG = FDUEG*PX**F53
        Chi = GX/PX**F83
        U = Css*Chi/(F1 + Css*Chi)
        W = sss0+U*(sss1+U*(sss2+U*(sss3+U*sss4)))
        Fscc=D/TX
        E = Fscc*W*EUEG
        F = E
        RSP = -RS/(F3*Px)
        ChiG = F1/PX**F83
        ChiP = -F83*Chi/PX
        dFsccP=Pt25*GX/(TX*PX**2)
        dFsccG=-Pt25/(TX*PX)
        dFsccT=Pt25*GX/(PX*TX**2)
        dUdChi=Css/((F1+Css*Chi)**2)
        dWdU=sss1+U*(F2*sss2+U*(F3*sss3+U*F4*sss4))
        dWdP=dWdU*dUdChi*ChiP
        dWdG=dWdU*dUdChi*ChiG 
        EUEGP = PotLC + PX*dLdS*RSP
        FP = (dFsccP*W*EUEG 
     $                 + Fscc*dWdP*EUEG
     $                 + Fscc*W*EUEGP)
        FG = (dFsccG*W*EUEG
     $                 + Fscc*dWdG*EUEG)

        FT = (dFsccT*W*EUEG)
       Endif

       Return
       End


